@@ -1,10 +1,8 @@
-# $Id: ConfigReader.pm 830 2005-09-17 23:40:59Z claco $
+# $Id: ConfigReader.pm 881 2005-10-03 23:01:55Z claco $
 package Handel::ConfigReader;
 use strict;
 use warnings;
-use vars qw(%Defaults);
-use Tie::Hash;
-use base 'Tie::StdHash';
+use vars qw(%Defaults $MOD_PERL);
 
 %Defaults = (
     HandelMaxQuantityAction => 'Adjust',
@@ -12,6 +10,26 @@ use base 'Tie::StdHash';
     HandelCurrencyFormat    => 'FMT_STANDARD'
 );
 
+BEGIN {
+    use Tie::Hash;
+    use base 'Tie::StdHash';
+
+    if (exists $ENV{MOD_PERL_API_VERSION} && $ENV{MOD_PERL_API_VERSION} == 2) {
+        require Apache2::RequestRec;
+        require Apache2::RequestUtil;
+        require Apache2::RequestIO;
+        require Apache2::ServerUtil;
+
+        $MOD_PERL = 2;
+    } elsif ($ENV{MOD_PERL}) {
+        require Apache;
+
+        $MOD_PERL = 1;
+    } else {
+        $MOD_PERL = 0;
+    };
+};
+
 sub new {
     my $class = shift;
     my %config;
@@ -32,11 +50,22 @@ sub FETCH {
     my $default = $Defaults{$key} || '';
     my $value   = '';
 
-    if ($ENV{MOD_PERL}) {
-        require Apache;
-        my $r = Apache->request;
+    if ($MOD_PERL == 2) {
+        my $c = eval {
+            Apache2::RequestUtil->request || Apache2::ServerUtil->server;
+        };
 
-        $value = $r->dir_config($key) || $ENV{$key} || $default;
+        if ($c) {
+            $value = eval{$c->dir_config($key)} || $ENV{$key} || $default;
+        };
+    } elsif ($MOD_PERL == 1) {
+        my $c = eval {
+            Apache->request || Apache->server;
+        };
+
+        if ($c) {
+            $value = $c->dir_config($key) || $ENV{$key} || $default;
+        };
     };
 
     if (!$value) {
